"""Fill annotator

Created by:
EOLE (http://eole.orion.education.fr)
Copyright (C) 2005-2018

Forked by:
Cadoles (http://www.cadoles.com)
Copyright (C) 2019-2021

distribued with GPL-2 or later license

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""
from ..utils import load_modules
from ..i18n import _
from ..error import DictConsistencyError


class FillAnnotator:
    """Fill annotator
    """
    def __init__(self,
                 objectspace,
                 eosfunc_file,
                 ):
        self.objectspace = objectspace
        if not hasattr(objectspace.space, 'constraints') or \
                not hasattr(self.objectspace.space.constraints, 'fill'):
            return
        self.functions = dir(load_modules(eosfunc_file))
        self.convert_fill()

    def convert_fill(self) -> None:
        """valid and manage <fill>
        """
        targets = []
        for fill in self.objectspace.space.constraints.fill:
            # test if it's redefined calculation
            if fill.target in targets:
                xmlfiles = self.objectspace.display_xmlfiles(fill.xmlfiles)
                msg = _(f'A fill already exists for the target of "{fill.target}" created '
                        f'in {xmlfiles}')
                raise DictConsistencyError(msg, 24)
            targets.append(fill.target)

            # test if the function exists
            if fill.name not in self.functions:
                xmlfiles = self.objectspace.display_xmlfiles(fill.xmlfiles)
                msg = _(f'cannot find fill function "{fill.name}" in {xmlfiles}')
                raise DictConsistencyError(msg, 25)

            # let's replace the target by the path
            fill.target, suffix = self.objectspace.paths.get_variable_path(fill.target,
                                                                           fill.namespace,
                                                                           )
            if suffix is not None:
                xmlfiles = self.objectspace.display_xmlfiles(fill.xmlfiles)
                msg = _(f'Cannot add fill function to "{fill.target}" only '
                        f'for the suffix "{suffix}" in {xmlfiles}')
                raise DictConsistencyError(msg, 26)

            # get the target variable
            variable = self.objectspace.paths.get_variable(fill.target)

            # create an object value
            value = self.objectspace.value(fill.xmlfiles)
            value.type = 'calculation'
            value.name = fill.name
            if variable.namespace == 'services':
                variable.default = value
            else:
                variable.value = [value]

            # manage params
            if not hasattr(fill, 'param'):
                continue
            self.convert_fill_param(fill)
            if fill.param:
                value.param = fill.param

    def convert_fill_param(self,
                           fill: "self.objectspace.fill",
                           ) -> None:
        """ valid and convert fill's param
        """
        param_to_delete = []
        for param_idx, param in enumerate(fill.param):
            if param.type == 'string' and not hasattr(param, 'text'):
                param.text = None
            if param.type == 'suffix':
                if hasattr(param, 'text'):
                    xmlfiles = self.objectspace.display_xmlfiles(fill.xmlfiles)
                    msg = _(f'"{param.type}" variables must not have a value in order '
                            f'to calculate "{fill.target}" in {xmlfiles}')
                    raise DictConsistencyError(msg, 28)
                if not self.objectspace.paths.variable_is_dynamic(fill.target):
                    xmlfiles = self.objectspace.display_xmlfiles(fill.xmlfiles)
                    msg = _('Cannot set suffix target to the none dynamic variable '
                            f'"{fill.target}" in {xmlfiles}')
                    raise DictConsistencyError(msg, 53)
            elif not hasattr(param, 'text'):
                xmlfiles = self.objectspace.display_xmlfiles(fill.xmlfiles)
                msg = _(f'All "{param.type}" variables must have a value in order '
                        f'to calculate "{fill.target}" in {xmlfiles}')
                raise DictConsistencyError(msg, 27)
            if param.type == 'variable':
                try:
                    path, suffix = self.objectspace.paths.get_variable_path(param.text,
                                                                            fill.namespace,
                                                                            )
                    param.text = self.objectspace.paths.get_variable(path)
                    if suffix:
                        param.suffix = suffix
                        family_path = self.objectspace.paths.get_variable_family_path(path)
                        param.family = self.objectspace.paths.get_family(family_path,
                                                                         param.text.namespace,
                                                                         )
                except DictConsistencyError as err:
                    if err.errno != 42 or not param.optional:
                        raise err
                    param_to_delete.append(param_idx)
        param_to_delete.sort(reverse=True)
        for param_idx in param_to_delete:
            fill.param.pop(param_idx)
